#include <gint/display.h>
#include <gint/keyboard.h>
#include "draw.h"
#include "shared_define.h"

#define BG_COLOR 0
#define DRAW_OFFSET_Y -24
#define DRAW_OFFSET_X -27

extern image_t img_player; //player texture, 12x12
extern image_t img_solid_0; //solid texture, 16x16
extern image_t img_solid_1; //solid texture, 16x16
extern image_t img_spike; //spike texture, 16x16
extern image_t img_bouncer; //bouncer texture, 16x16
extern image_t img_ice; //ice texture, 16x8
extern image_t img_blue; //blue bloc texture, 16x16
extern image_t img_blue_dot; //off blue bloc texture, 16x16
extern image_t img_red; //red bloc texture, 16x16
extern image_t img_red_dot; //off red bloc texture, 16x16
extern image_t img_exit; //exit texture, 16x16
extern image_t img_water; //water texture, 16x16
extern image_t img_semi_solid; //semi solid texture, 16x16
//animated textures (elevator)
extern image_t img_teleporter_0; //teleporter 0 texture, 16x16
extern image_t img_teleporter_1; //teleporter 1 texture, 16x16
extern image_t img_elevator;

void draw_player(int x, int y)
{
	dimage(x + DRAW_OFFSET_X, y + DRAW_OFFSET_Y, &img_player);
}

void draw_level(char level[], unsigned int step, char polarity, int *start_x,
int *start_y, int tp_positions[])
{
	int const img_elevator_frame_count = img_elevator.width / 16;
	int const img_teleporter_0_frame_count = img_teleporter_0.width / 16;
	int const img_teleporter_1_frame_count = img_teleporter_1.width / 16;
	dclear(BG_COLOR);
	unsigned int i = 0;
	unsigned int x = 0;
	unsigned int y = 0;
	while (i != LEVEL_WIDTH*16)
	{
		switch (level[i])
		{
			case '0':
				dimage(x + DRAW_OFFSET_X, y + DRAW_OFFSET_Y, &img_solid_0);
				break;
			case '1':
				dimage(x + DRAW_OFFSET_X, y + DRAW_OFFSET_Y, &img_solid_1);
				break;
			case 'r':
				if (!polarity) dimage(x + DRAW_OFFSET_X, y + DRAW_OFFSET_Y, &img_red);
				else dimage(x + DRAW_OFFSET_X, y + DRAW_OFFSET_Y, &img_red_dot);
				break;
			case 'b':
				if (polarity) dimage(x + DRAW_OFFSET_X, y + DRAW_OFFSET_Y, &img_blue);
				else dimage(x + DRAW_OFFSET_X, y + DRAW_OFFSET_Y, &img_blue_dot);
				break;
			case 'v':
				dimage(x + DRAW_OFFSET_X, y + DRAW_OFFSET_Y, &img_spike);
				break;
			case '*':
				dimage(x + DRAW_OFFSET_X, y + DRAW_OFFSET_Y, &img_bouncer);
				break;
			case '~':
				dimage(x + DRAW_OFFSET_X, y + DRAW_OFFSET_Y, &img_solid_0);
				dimage(x + DRAW_OFFSET_X, y + DRAW_OFFSET_Y, &img_ice);
				break;
			case '/':
				dimage(x + DRAW_OFFSET_X, y + DRAW_OFFSET_Y, &img_semi_solid);
				break;
			case '^':
        dsubimage(x + DRAW_OFFSET_X, y + DRAW_OFFSET_Y, &img_elevator,
          ((step / 2) % img_elevator_frame_count) * 16, 0, 16, 16, DIMAGE_NONE);
				break;
			case 'S':
				erase_tile(x, y, level);
				*start_x = x + 2;
				*start_y = y + 4;
				break;
			case 'E':
				dimage(x + DRAW_OFFSET_X, y + DRAW_OFFSET_Y, &img_exit);
				break;
			case 'w': //water
				dimage(x + DRAW_OFFSET_X, y + DRAW_OFFSET_Y, &img_water);
				break;
			case 't': //teleporter 0
        dsubimage(x + DRAW_OFFSET_X, y + DRAW_OFFSET_Y, &img_teleporter_0,
          (step % img_teleporter_0_frame_count) * 16, 0, 16, 16, DIMAGE_NONE);
				tp_positions[0] = x + 2;
				tp_positions[1] = y + 2;
				break;
			case 'T': //teleporter 1
        dsubimage(x + DRAW_OFFSET_X, y + DRAW_OFFSET_Y, &img_teleporter_1,
          (step % img_teleporter_1_frame_count) * 16, 0, 16, 16, DIMAGE_NONE);
				tp_positions[2] = x + 2;
				tp_positions[3] = y + 2;
				break;
		}
		x += 16;
		if (x == 16*LEVEL_WIDTH)
		{
			x = 0;
			y += 16;
		}
		i++;
	}
}

void erase_tile(int x, int y, char level[])
{
	x = (int)(x/16);
	y = (int)(y/16);
	level[x + y * LEVEL_WIDTH] = '.';
	x *= 16;
	y *= 16;
}

void draw_timer(unsigned int step)
{
	dprint(0, 0, C_WHITE, C_BLACK, "%u.%02u", step/FPS, step%FPS);
}

void just_breathe(unsigned int step)
{
	int x = 0;
	int y = 0;
	int xspd = 1;
	int yspd = 1;
	while (!keydown_any(KEY_MENU, KEY_EXIT, 0))
	{
		dclear(0);
		dtext(x, y, "Thank you for playing", C_WHITE, C_BLACK);
		dprint(x, y + 12, C_WHITE, C_BLACK, "%u.%02u", step/FPS, step%FPS);
		dupdate();
		x += xspd;
		y += yspd;
		if (x == 236 || !x) xspd = 0 - xspd;
		if (y == 203 || !y) yspd = 0 - yspd;
		clearevents();
	}
	{
		unsigned char timeout = 50;
		while(timeout)
		{
			timeout--;
			dclear(0);
		}
	}
}
